package com.agiledeveloper;

import rx.Observable;
import rx.Subscriber;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MovieServer {
  List<String> cachedMovieNames;

  public Observable<String> getDirectors(final List<String> movieNames) {
    return Observable.create(
        new Observable.OnSubscribe<String>() {
          @Override
          public void call(final Subscriber<? super String> subscriber) {
            if(cachedMovieNames == movieNames)
              pushMovieInfo(subscriber, movieNames);
            else {
              cachedMovieNames = movieNames;
              pushMovieInfoAsync(subscriber, movieNames);
            }
          }
        });
  }

  private void pushMovieInfoAsync(final Subscriber<? super String> subscriber, final List<String> movieNames) {
    ExecutorService executorService = Executors.newFixedThreadPool(10);

    executorService.submit(
        new Runnable() {
          @Override
          public void run() {
            pushMovieInfo(subscriber, movieNames);
          }
        });

    executorService.shutdown();
  }

  private void pushMovieInfo(Subscriber<? super String> subscriber, List<String> movieNames) {
    for (String movieName : movieNames) {
      String director = MovieInfo.getMovieInfo(movieName).get("Director");
      subscriber.onNext(movieName + "---" + director);
    }
  }
}